using System;
using System.ComponentModel;
using Plux;
using Plux.Framework;
#if !SILVERLIGHT
using System.Drawing;
using System.Windows.Forms;

#else
using Plux.Silverlight;
using System.Windows.Input;
using System.Windows.Controls;
#endif

#if !SILVERLIGHT
[Extension("Console", OnCreated = "OnCreated", OnReleased = "OnReleased")]
[Plug("Startup")]
[ParamValue("ExecuteInMainThread", true)]
[Plug("Plux.View", OnPlugged = "View_Plugged")]
[Plug("Plux.Tool", OnPlugged = "Tool_Plugged")]
[Plug("Plux.PropertyGroup")]
[ParamValue("Name", "Console")]
[ParamValue("Kind", ViewKind.Tool)]
[ParamValue("OrderIndex", 0.9f)]
[ParamValue("Icon", "console.ico")]
[Slot("Plux.Preferences", OnPlugged = "Preferences_Plugged",
    OnUnplugged = "Preferences_Unplugged")]
[Slot("Plux.CommandInterpreter", Multiple = false, LazyLoad = true)]
public class Console : IStartup, IFlipFormChild, ITool, IView, IPropertyGroup {
#else
  public class Console : IView {
#endif

  private PlugInfo viewPlug;
  private SlotInfo cmdInterpreterSlot;
  private int curComplIndex;
  private string cmdStart;
  private string[] complOptions;
  private ExtensionInfo ext;

#if !SILVERLIGHT
  private readonly Font defaultFont =
      new Font("Lucida Console", 9.75F, FontStyle.Regular, GraphicsUnit.Point, 0);
  private readonly ConsoleForm form;
  private bool shutdownOnCloseForm;
  private PlugInfo toolPlug;
  public ConsoleUC ConsoleUC { get { return form.consoleUC; } }
#else
    private readonly ConsoleUC consoleUC = new ConsoleUC();
    public ConsoleUC ConsoleUC {
      get { return consoleUC; }
    }
#endif

#if !SILVERLIGHT
  public Console() {
    shutdownOnCloseForm = true;
    form = new ConsoleForm();
    form.Closed += Form_Closed;
    ConsoleUC.ShellTextFont = defaultFont;
  }

  [Category("Console")]
  public Font Font { get { return form.consoleUC.ShellTextFont; } set { form.consoleUC.ShellTextFont = value; } }
  [Category("Console"), DefaultValue(typeof(Color), "LightGray")]
  public Color TextColor { get { return form.consoleUC.ShellTextForeColor; } set { form.consoleUC.ShellTextForeColor = value; } }
  [Category("Console"), DefaultValue(typeof(Color), "Black")]
  public Color BackColor { get { return form.consoleUC.ShellTextBackColor; } set { form.consoleUC.ShellTextBackColor = value; } }

  #region IFlipFormChild Members

  [Browsable(false)]
  public Form Form { get { return form; } }
  [Browsable(false)]
  public IFlipForm FlipFormParent { set { } }

  #endregion

  #region IStartup Members

  public void Run() {
    if (viewPlug.PlugTypeInfo.RegisteredInSlots.Count > 0 ||
        toolPlug.PlugTypeInfo.RegisteredInSlots.Count > 0) {
      return;
    }
    Runtime.TaskQueue.InvokeOnEmptied(Show);
  }

  #endregion

  #region ITool Members

  public void Dispose() {
    form.Dispose();
  }

  public void Show() {
    form.Show();
    form.Activate();
  }

  #endregion

  [Browsable(false)]
  public Control Control { get { return form.consoleUC; } }

#else


    public Control Control {
      get { return consoleUC; }
    }

#endif

  [Category("Console")]
  public string Name { get { return "Console"; } }

  private ICommandInterpreter Interpreter {
    get {
      return cmdInterpreterSlot.PluggedPlugInfos.Count > 0
          ? cmdInterpreterSlot.PluggedPlugInfos[0].ExtensionInfo.Object as ICommandInterpreter
          : null;
    }
  }

#if !SILVERLIGHT

  public void Preferences_Plugged(object sender, PlugEventArgs args) {
    var prefs = new Preferences(args);
    if (ext.PlugInfos["Startup"].PluggedInSlots.Count > 1) {
      prefs.RestoreWindow(form);
    }
    Font = (Font)prefs.Get("Font", Font);
    TextColor = (Color)prefs.Get("TextColor", TextColor);
    BackColor = (Color)prefs.Get("BackColor", BackColor);
  }

  public void Preferences_Unplugged(object sender, PlugEventArgs args) {
    var prefs = new Preferences(args);
    if (ext.PlugInfos["Startup"].PluggedInSlots.Count > 1) {
      prefs.SaveWindow(form);
    }
    prefs.Set("Font", Font);
    prefs.Set("TextColor", TextColor);
    prefs.Set("BackColor", BackColor);
  }
#endif

  private void Shell_CommandEntered(object sender, CommandEventArgs e) {
    if (String.IsNullOrEmpty(e.Command)) {
      return;
    }
    if (Interpreter == null) {
      if (!Runtime.AutoRegister) {
        cmdInterpreterSlot.Register();
      }
      SlotInfoEx.CreateAndPlug(cmdInterpreterSlot);
      if (Interpreter == null) {
        ConsoleUC.WriteLine("  Command '{0}' not found - command interpreter is missing", e.Command);
        return;
      }
    }
    Interpreter.StateChanged += Interpreter_StateChanged;
    //Interpreter.ExecuteAsync(e.Command);
    ConsoleUC.WriteLine(Interpreter.Execute(e.Command));
  }

  private void Interpreter_StateChanged(object sender, EventArgs e) {
    ICommandInterpreter interpreter = (ICommandInterpreter)sender;
    if (interpreter.State != CommandState.Completed) {
      return;
    }
    interpreter.StateChanged -= Interpreter_StateChanged;
    ConsoleUC.WriteLine(interpreter.Output);
  }

  private void CompleteCommand(bool backward) {
    if (complOptions == null || complOptions.Length == 0) {
      return;
    }
    if (backward) {
      curComplIndex--;
    } else {
      curComplIndex++;
    }
    if (curComplIndex < 0) {
      curComplIndex = complOptions.Length - 1;
    } else if (curComplIndex == complOptions.Length) {
      curComplIndex = 0;
    }
    ConsoleUC.CurrentLine = cmdStart + complOptions[curComplIndex];
  }

  public void OnCreated(object sender, ExtensionEventArgs args) {
    ConsoleUC.CommandEntered += Shell_CommandEntered;
    ConsoleUC.KeyDown += ShellKeyDown;
    ConsoleUC.InitText = String.Format(
        "{0}{2}{1}{2}",
        Runtime.VersionInfo(),
        "Runtime is launched.",
        Environment.NewLine);
    ext = args.ExtensionInfo;
    viewPlug = ext.PlugInfos["Plux.View"];
    cmdInterpreterSlot = ext.SlotInfos["Plux.CommandInterpreter"];
#if !SILVERLIGHT
    toolPlug = ext.PlugInfos["Plux.Tool"];
#endif
  }

#if SILVERLIGHT
    private void ShellKeyDown(object sender, KeyEventArgs e) {
      if (e.Key == Key.Tab) {
        if (complOptions == null) {
          if (Interpreter == null) {
            SlotInfoEx.CreateAndPlug(cmdInterpreterSlot);
            if (Interpreter == null) {
              return;
            }
          }
          cmdStart = ConsoleUC.CurrentLine;
          complOptions = Interpreter.GetCommandOptions(ref cmdStart);
        }
        CompleteCommand(Keyboard.Modifiers == ModifierKeys.Shift);
      } else if (Keyboard.Modifiers == 0) {
        complOptions = null;
        curComplIndex = -1;
        if (e.Key == Key.Escape) {
          ConsoleUC.CurrentLine = String.Empty;
        }
      }
    }
#else
  private void ShellKeyDown(object sender, KeyEventArgs e) {
    if (e.KeyCode == Keys.Tab) {
      if (complOptions == null) {
        if (Interpreter == null) {
          SlotInfoEx.CreateAndPlug(cmdInterpreterSlot);
          if (Interpreter == null) {
            return;
          }
        }
        cmdStart = ConsoleUC.CurrentLine;
        complOptions = Interpreter.GetCommandOptions(ref cmdStart);
      }
      CompleteCommand(e.Shift);
    } else if (!(e.Shift || e.Alt || e.Control)) {
      complOptions = null;
      curComplIndex = -1;
      if (e.KeyCode == Keys.Escape) {
        ConsoleUC.CurrentLine = String.Empty;
      }
    }
  }

  private void Form_Closed(object sender, EventArgs e) {
    if (shutdownOnCloseForm) {
      ExitAction.Do(this);
    }
    ext.Release();
  }

  public void OnReleased(object sender, ExtensionEventArgs args) {
    shutdownOnCloseForm = false;
    form.Hide();
  }

  public void View_Plugged(object sender, PlugEventArgs args) {
    if(!Runtime.IsCompositionAllowed) return;
    toolPlug.Unplug();
    foreach (SlotInfo si in viewPlug.PluggedInSlots) {
      if (si == args.SlotInfo) {
        continue;
      }
      si.Unplug(viewPlug);
    }
  }

  public void Tool_Plugged(object sender, PlugEventArgs args) {
    shutdownOnCloseForm = false;
    viewPlug.Unplug();
    foreach (SlotInfo si in toolPlug.PluggedInSlots) {
      if (si == args.SlotInfo) {
        continue;
      }
      si.Unplug(toolPlug);
    }
  }

  public bool ShouldSerializeFont() {
    return form.consoleUC.ShellTextFont != defaultFont;
  }

  public void ResetFont() {
    form.consoleUC.ShellTextFont = defaultFont;
  }
#endif
}