using System;
using System.Drawing;
using System.Windows.Forms;
using Plux;
using Plux.Framework;
using SelectionMode=Plux.SelectionMode;


[Extension("Workbench", OnCreated = "OnCreated", OnReleased = "OnReleased",
    Singleton = true)]

[Plug("Startup")]
[ParamValue("ExecuteInMainThread", true)]
[ParamValue("UnplugOnExit", false)]

[Plug("Plux.MdiLayout")]

[Slot("Plux.MenuStrip", Unique = true,
    OnPlugged = "Menu_Plugged", OnUnplugged = "Menu_Unplugged")]

[Slot("Plux.Action", Multiple = true, LazyLoad = true)]

[Slot("Plux.View", Multiple = true, Unique = true, LazyLoad = true, OnRegistering = "View_Registering", OnPlugged = "View_Plugged", OnUnplugged = "View_Unplugged")]

[Slot("Plux.Tool", Multiple=true, Unique=true, LazyLoad=true, OnPlugged="Tool_Plugged")]

[Slot("Workbench.Sidebar", DerivedFrom = "Plux.Control", Unique = true,
    AutoPlug = false,
    OnPlugged = "Sidebar_Plugged", OnUnplugged = "Sidebar_Unplugged")]

[Slot("Plux.ToolStrip", Unique = true, AutoPlug = false,
    OnPlugged = "ToolStrip_Plugged", OnUnplugged = "ToolStrip_Unplugged")]

[Slot("Plux.Preferences",
    OnPlugged = "Preferences_Plugged",
    OnUnplugged = "Preferences_Unplugged")]
[AllowRegister("04FFF94017C28A0C")]
[AllowPlug("04FFF94017C28A0C")]

public partial class MainForm : Form, IStartup, IMdiLayout {
  private bool _userClickedClose = true;
  private SlotInfo actionSlot = null;
  private MdiManager mdi = null;
  private MenuManager menu = null;
  private ToolStripManager toolStrip = null;
  private SlotInfo viewSlot = null;
  private SlotInfo startupSlot;
  private SlotInfo toolSlot;

  public MainForm() {
    Application.EnableVisualStyles();
    InitializeComponent();
    sideBarPanel.Visible = false;
    sideBarSplitter.Visible = false;
    toolbarPanel.Visible = false;
    KeyPreview = true;
    KeyDown += MainForm_KeyDown;
  }

  private void MainForm_FormClosing(object sender, FormClosingEventArgs e) {
    if (_userClickedClose) {
      ExitAction.Do(this);
    }
  }

  public void OnCreated(object s, ExtensionEventArgs args) {
    viewSlot = Runtime.GetSlotInfo(this, "Plux.View");
    actionSlot = Runtime.GetSlotInfo(this, "Plux.Action");
    toolSlot = Runtime.GetSlotInfo(this, "Plux.Tool");
    viewSlot.SelectionMode = SelectionMode.Single;
    mdi = new MdiManager(viewSlot, this,
        new Icon(typeof(MainForm), "unknown.ico"));
    startupSlot = Runtime.Repository.SlotInfos["Startup"];
    startupSlot.Registering += Startup_Registering;
    if (Runtime.Arguments.ContainsKey("Console")) {
      ActivateConsoleMode();
    }
    Show();
  }

  private void ActivateConsoleMode() {
    viewSlot.AutoRegister = false;
    actionSlot.AutoRegister = false;
    Runtime.TaskQueue.InvokeOnEmptied(RegisterMenu);
    Runtime.Repository.PluginAdded += RegisterConsole;
  }

  private static int consoleModePlugins = 2;

  private void RegisterMenu() {
    var ext = Runtime.GetExtensionInfo(this);
    Runtime.Repository.ExtensionTypeInfos["FileExit"].RegisterPlugs(ext);
  }

  private void RegisterConsole(object s, PluginEventArgs args) {
    var ext = Runtime.GetExtensionInfo(this);
    if (args.PluginInfo.Name.Equals("Console.dll")) {
      var consoleType = Runtime.Repository.ExtensionTypeInfos["Console"];
      consoleType.RegisterPlugs(ext);
      var console = consoleType.CreateUniqueExtension();
      console.PlugPlugs(ext);
      consoleModePlugins--;
    } else if (args.PluginInfo.Name.Equals("HotRibbon.dll")) {
      Runtime.Repository.ExtensionTypeInfos["ViewToolbar"].RegisterPlugs(ext);
      Runtime.Repository.ExtensionTypeInfos["ViewSeparator"].RegisterPlugs(ext);
      consoleModePlugins--;
    }
    if (consoleModePlugins == 0) {
      Runtime.Repository.PluginAdded += RegisterConsole;
    }
  }

  public void OnReleased(object s, ExtensionEventArgs args) {
    _userClickedClose = false;
    startupSlot.Registering -= Startup_Registering;
    Hide();
  }

  private static void Startup_Registering(object s, CancelRegisterEventArgs args) {
    ExtensionTypeInfo extType = args.PlugTypeInfo.ExtensionTypeInfo;
    args.Cancel = (extType.PlugTypeInfos["Plux.View"] != null);
  }

  private void MainForm_KeyDown(object sender, KeyEventArgs e) {
    if(e.Control) {
      switch(e.KeyCode) {
        case Keys.R:
          QuickDialog.Register(this);
          e.Handled = true;
          break;
        case Keys.D:
          QuickDialog.Deregister(this);
          e.Handled = true;
          break;
        case Keys.P:
          QuickDialog.Plug(this);
          e.Handled = true;
          break;
        case Keys.U:
          QuickDialog.Unplug(this);
          e.Handled = true;
          break;
        case Keys.S:
          QuickDialog.Select(this);
          e.Handled = true;
          break;
        case Keys.F1:
          QuickDialog.Save(this);
          e.Handled = true;
          break;
        case Keys.F2:
          QuickDialog.Restore(this);
          e.Handled = true;
          break;
        case Keys.A:
          QuickDialog.Save(this);
          e.Handled = true;
          break;
      }
    }
  }

  #region Plug: Startup

  public void Run() {}

  #endregion

  #region Slot: Sidebar

  public void Sidebar_Plugged(object s, PlugEventArgs args) {
    sideBarSplitter.Visible = true;
    // Reihenfolge beachten: zuerst Splitter, dann Panel sichtbar 
    // machen, sonst kann man Splitter nicht bewegen!
    sideBarPanel.Visible = true;
    Control ctl = ((IControl) args.Extension).Control;
    sideBarPanel.Width = ctl.Width;
    sideBarPanel.Controls.Add(ctl);
  }

  public void Sidebar_Unplugged(object s, PlugEventArgs args) {
    sideBarPanel.Visible = false;
    sideBarSplitter.Visible = false;
    sideBarPanel.Controls.Remove(((IControl) args.Extension).Control);
  }

  #endregion

  #region Slot: Toolbar

  public void ToolStrip_Plugged(object s, PlugEventArgs args) {
    toolbarPanel.Visible = true;
    toolStrip = new ToolStripManager(args.PlugInfo, viewSlot, actionSlot);
    Control toolstrip = ((IToolStrip) args.Extension).Control;
    toolbarPanel.Controls.Add(toolstrip);
  }

  public void ToolStrip_Unplugged(object s, PlugEventArgs args) {
    toolbarPanel.Visible = false;
    toolbarPanel.Controls.Remove(((IToolStrip) args.Extension).Control);
    toolStrip.Close();
    toolStrip = null;
  }

  #endregion

  #region Slot: View

  public void View_Registering(object s, CancelRegisterEventArgs args) {
    ExtensionTypeInfo extType = args.PlugTypeInfo.ExtensionTypeInfo;
    args.Cancel = (extType.Singleton && extType.GetSharedExtension(false) != null);
  }

  public void View_Plugged(object s, PlugEventArgs args) {
    mdi.CreateChildForm(args.PlugInfo);
  }

  public void View_Unplugged(object s, PlugEventArgs args) {
    mdi.CloseChildForm(args.PlugInfo);
  }

  #endregion

  #region Slot: MenuStrip

  public void Menu_Plugged(object sender, PlugEventArgs args) {

    menu = new MenuManager((IMenuStrip)args.Extension, viewSlot, actionSlot, toolSlot);
    MainMenuStrip = ((IMenuStrip) args.Extension).Control;
    MainMenuStrip.Dock = DockStyle.Top;
    MainMenuStrip.TabIndex = 0;
    MainMenuStrip.Location = new Point(0, 0);
    Controls.Add(MainMenuStrip);
  }

  public void Menu_Unplugged(object sender, PlugEventArgs args) {
    Controls.Remove(((IMenuStrip) args.Extension).Control);
    menu.CloseMenu();
    menu = null;
  }

  #endregion

  #region Slot: Preferences

  public void Preferences_Plugged(object sender, PlugEventArgs args) {
    var pref = new Preferences(args);
    pref.RestoreWindow(this);
    mdi.RestoreSettings(pref);
  }

  public void Preferences_Unplugged(object sender, PlugEventArgs args) {
    var pref = new Preferences(args);
    pref.SaveWindow(this);
    mdi.SaveSettings(pref);
  }

  #endregion

  public void Tool_Plugged(object s, PlugEventArgs args) {
    args.GetExtension<ITool>().Show();
  }

  public void ArrangeIcons() {
    LayoutMdi(MdiLayout.ArrangeIcons);
  }

  public void MinimizeAll() {
    foreach (Form child in MdiChildren)
      child.WindowState = FormWindowState.Minimized;
    ArrangeIcons();
  }

  public void RestoreAll() {
    foreach(Form child in MdiChildren)
      child.WindowState = FormWindowState.Normal;
  }

  public void Cascade() {
    LayoutMdi(MdiLayout.Cascade);
  }

  public void TileHorizontal() {
    LayoutMdi(MdiLayout.TileHorizontal); 
  }

  public void TileVertical() {
    LayoutMdi(MdiLayout.TileVertical);
  }
}