using System;
using System.Collections.Generic;
using System.Linq;
using Plux;
using Initializer = Plux2010.Desktop.Initializer;
using Runtime = Plux.Runtime;
using VerbosityLevel = Plux.VerbosityLevel;


public static class Program {
  private static readonly Dictionary<string, string> shortForms
      = new Dictionary<string, string>();

  static Program() {
    shortForms.Add("ver", "version");
    shortForms.Add("?", "help");
    shortForms.Add("h", "help");
    shortForms.Add("v", "verbosity");
    shortForms.Add("q", "quiet");
    shortForms.Add("m", "minimal");
    shortForms.Add("n", "normal");
    shortForms.Add("d", "detailed");
    shortForms.Add("diag", "diagnostic");
    shortForms.Add("dis", "discovery");
    shortForms.Add("b", "base");
    shortForms.Add("l", "logfile");
    shortForms.Add("nl", "nologo");
    shortForms.Add("args", "arguments");
    shortForms.Add("cad", "createappdomain");
    shortForms.Add("2010", "2010");
    shortForms.Add("trace", "trace");
  }

  [STAThread]
  public static void Main(string[] args) {
    Runtime.VersionInfo();

    Dictionary<string, string> options = GetOptions(args);

    KeyValuePair<string, string>[] arguments2010 = GetArguments2010(options);
    Dictionary<string, string> arguments = GetArguments(arguments2010);

    if (options.ContainsKey("version")) {
      Console.WriteLine(Runtime.VersionInfo());
      return;
    }

    if (options.ContainsKey("help")) {
      Console.WriteLine(Runtime.VersionInfo());
      WriteHelp();
      return;
    }

    bool noLogo = options.ContainsKey("nologo");
    if (!options.ContainsKey("2010") && !noLogo) {
      Console.WriteLine(Runtime.VersionInfo());
    }

    VerbosityLevel verbosity = VerbosityLevel.Normal;
    if (options.ContainsKey("verbosity")) {
      verbosity = (VerbosityLevel) Enum.Parse(typeof(VerbosityLevel), options["verbosity"], true);
    }

    string[] discoveryPaths = null;
    if (options.ContainsKey("discovery")) {
      discoveryPaths = options["discovery"].Split(new[] { ';' },
          StringSplitOptions.RemoveEmptyEntries);
    }

    string[] baseDirectories;
    if (!options.ContainsKey("base")) {
      baseDirectories = new[] { AppDomain.CurrentDomain.BaseDirectory };
    } else {
      baseDirectories =
          options["base"].Split(new[] { ';' },
              StringSplitOptions.RemoveEmptyEntries);
    }

    string logfile = options.ContainsKey("logfile") ? options["logfile"] : string.Empty;
    bool createAppDomain = options.ContainsKey("createappdomain");
    bool trace = options.ContainsKey("trace");

    if (options.ContainsKey("2010")) {
      Plux2010.VerbosityLevel verbosityLevel = (Plux2010.VerbosityLevel) (int) verbosity;
      Initializer.Start(
        bootstrapPaths: discoveryPaths,
        assemblyDirectories: baseDirectories,
        arguments: arguments2010,
        verbosityLevel: verbosityLevel,
        noLogo: noLogo,
        logfile: logfile,
        trace: trace,
        createAppDomain: createAppDomain);
    } else {
      ClientRuntime runtime = ClientRuntime.Create(discoveryPaths, baseDirectories, verbosity, arguments, logfile, createAppDomain);
      runtime.Start();
    }
  }

  private static Dictionary<string, string> GetOptions(IEnumerable<string> args) {
    Dictionary<string, string> options = new Dictionary<string, string>();
    foreach (string arg in args) {
      if (arg.Length > 1 && arg.IndexOfAny(new[] { '/', '-' }, 0, 1) == 0) {
        string[] parts = arg.Substring(1).Split(':');
        string name = shortForms.ContainsKey(parts[0])
            ? shortForms[parts[0]]
            : parts[0];
        name = name.ToLower();
        string value = string.Empty;
        if (parts.Length > 1) {
          value = shortForms.ContainsKey(parts[1])
              ? shortForms[parts[1]]
              : parts[1];
        }
        options[name] = value;
        if (!shortForms.ContainsValue(name)) {
          string errMessage =
              value != String.Empty
                  ? String.Format("Invalid start parameter: option \"/{0}:{1}\" not found", name, value)
                  : String.Format("Invalid start parameter: option \"/{0}\" not found", name);
          PrintErrorAndQuit(errMessage);
        }
      } else {
        PrintErrorAndQuit(String.Format("Invalid start parameter: {0}\n  Parameter must start with '/' or '-'.", arg));
      }
    }

    if (options.ContainsKey("logfile") && String.IsNullOrEmpty(options["logfile"])) {
      PrintErrorAndQuit("Filename of logfile must not be empty.");
    }

    return options;
  }

  private static KeyValuePair<string, string>[] GetArguments2010(Dictionary<string, string> options) {
    if (!options.ContainsKey("arguments")) {
      return new KeyValuePair<string, string>[0];
    }
    try {
      string[] keyValuePairs =
          options["arguments"].Split(new[] { ';' }, StringSplitOptions.RemoveEmptyEntries);
      List<KeyValuePair<string, string>> arguments = new List<KeyValuePair<string, string>>(keyValuePairs.Length);
      foreach (string keyValuePair in keyValuePairs) {
        string[] pair = keyValuePair.Split('=');
        arguments.Add(new KeyValuePair<string, string>(pair[0], pair[1]));
      }
      return arguments.ToArray();
    } catch {
      throw new ArgumentException("invalid argument string.\n  args: " + options["arguments"]);
    }
  }

  private static Dictionary<string, string> GetArguments(IEnumerable<KeyValuePair<string, string>> keyValuePairs) {
    return keyValuePairs.ToDictionary(pair => pair.Key, pair => pair.Value);
  }

  private static void PrintErrorAndQuit(string errorMessage) {
    ConsoleColor oldColor = Console.ForegroundColor;
    Console.ForegroundColor = ConsoleColor.Red;
    Console.WriteLine(errorMessage);
    Console.ForegroundColor = oldColor;
    Console.WriteLine("Press ENTER to exit...");
    Console.ReadLine();
    Environment.Exit(1);
  }

  private static void WriteHelp() {
    Console.WriteLine(
        @"
Syntax:              Runtime.exe [options]

Description:         Launches the Plux.NET runtime.

Switches:

  /help              Display this usage message. (Short form: /? or /h)

  /nologo            Do not display the startup banner and copyright message.

  /version           Display version information only. (Short form: /ver)

  /process:<name>    Use process with given name.

  /processid:<id>    Use process with given id.

  /verbosity:<level> Display this amount of information in the event log.
                     The available verbosity levels are: q[uiet], m[inimal],
                     n[ormal], d[etailed], and diag[nostic]. (Short form: /v)
                     Example:
                       /verbosity:quiet
  
  /discovery:<paths> Files and directories for initial discovery of contracts
                     and plugins. Separate paths by ';'. Default are all 
                     assemblies in the base directory of the application. 
                     (Short form: /dis)
                     Example:
                       /dis:Cerberus.dll;contracts\;plugins\Monitor.dll;
      
  /base:<paths>      Directories which are registered in the ApplicationBase to
                     discover plugins and contracts from there. Seperate paths
                     by ';'. Default is the base directory of the application. 
                     Example:
                       /base:.\;plugins\;contracts\

  /logfile:<path>    The Logger writes the messages in the specified file.
                     Example:
                       /logfile:.\logfile.txt

  /createAppDomain   Create a new app domain on start up."
        );
  }
}